@@ -6,7 +6,7 @@ WWW::Search::Null::Error - class for testing WWW::Search clients
 
 =head1 SYNOPSIS
 
-=for example begin
+=begin example
 
   require WWW::Search;
   my $oSearch = new WWW::Search('Null::Error');
@@ -15,7 +15,7 @@ WWW::Search::Null::Error - class for testing WWW::Search clients
   my $oResponse = $oSearch->response;
   # You get an HTTP::Response object with a code of 500
 
-=for example end
+=end example
 
 =for example_testing
 is($oResponse->code, 500, 'did not get a 500 HTTP::Response');
@@ -38,16 +38,17 @@ package WWW::Search::Null::Error;
 
 use strict;
 
-use vars qw( @ISA );
+use vars qw( @ISA $VERSION );
 @ISA = qw( WWW::Search );
+$VERSION = do { my @r = (q$Revision: 1.4 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 
-sub native_setup_search
+sub _native_setup_search
   {
   my($self, $native_query, $native_opt) = @_;
   } # native_setup_search
 
 
-sub native_retrieve_some
+sub _native_retrieve_some
   {
   my $self = shift;
   my $response = new HTTP::Response(500,
