@@ -1,11 +1,11 @@
 # SearchResult.pm
 # by John Heidemann
 # Copyright (C) 1996 by USC/ISI
-# $Id: SearchResult.pm,v 1.10 2003-01-22 08:39:45-05 kingpin Exp kingpin $
+# $Id: SearchResult.pm,v 2.61 2004/05/24 00:14:18 Daddy Exp Daddy $
 #
 # Copyright (c) 1996 University of Southern California.
-# All rights reserved.                                            
-#                                                                
+# All rights reserved.
+#
 # Redistribution and use in source and binary forms are permitted
 # provided that the above copyright notice and this paragraph are
 # duplicated in all such forms and that any documentation, advertising
@@ -14,7 +14,7 @@
 # Southern California, Information Sciences Institute.  The name of the
 # University may not be used to endorse or promote products derived from
 # this software without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 # WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 # MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -61,7 +61,7 @@ package WWW::SearchResult;
 require LWP::MemberMixin;
 @ISA = qw(LWP::MemberMixin);
 use Carp ();
-$VERSION = '2.06';
+$VERSION = do { my @r = (q$Revision: 2.61 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 
 =head2 new
 
@@ -72,7 +72,7 @@ To create a new WWW::SearchResult, call
 =cut
 
 sub new
-  { 
+  {
   my $class = shift;
   my $self = bless { }, $class;
   $self->{urls} = ();
@@ -90,59 +90,102 @@ Every result is required to have at least one URL.
 
 =cut
 
-sub url {
-    my($self) = shift @_;
-    return ${$self->{urls}}[0] if ($#_ == -1);
-    unshift @{$self->{urls}}, $_[0];
+sub url
+  {
+  my $self = shift;
+  if (@_ < 1)
+    {
+    # No arguments, just return the current value:
     return ${$self->{urls}}[0];
-}
+    } # if no args
+  unshift @{$self->{urls}}, $_[0];
+  return $self->{urls}->[0];
+  } # url
 
-sub _elem_array {
-    my($self) = shift @_;
-    my($elem) = shift @_;
-    return wantarray ? @{$self->{$elem}} : $self->{$elem}
-        if ($#_ == -1);
-    if (ref($_[0])) {
-        $self->{$elem} = $_[0];
-    } else {
-	$self->{$elem} = ();
-	push @{$self->{$elem}}, @_;
-    };
-    # always return array refrence
-    return $self->{$elem};
-}
+sub _elem_array
+  {
+  my $self = shift;
+  my $elem = shift;
+  if (@_ < 1)
+    {
+    # No arguments
+    return wantarray ? @{$self->{$elem}} : $self->{$elem};
+    } # if
+  if (ref($_[0]))
+    {
+    # Trying to assign an arrayref:
+    $self->{$elem} = $_[0];
+    }
+  else
+    {
+    # Trying to set to a scalar (or list of scalars); make sure it's
+    # an array even if they give one element:
+    $self->{$elem} = undef;
+    push @{$self->{$elem}}, @_;
+    }
+  # Always return array refrence
+  return $self->{$elem};
+  } # _elem_array
 
-sub _add_elem_array {
-    my($self) = shift @_;
-    my($elem) = shift @_;
-    push(@{$self->{$elem}}, @_);
-};
+sub _add_elem_array
+  {
+  my $self = shift;
+  my $elem = shift;
+  # No matter how many they're adding:
+  push(@{$self->{$elem}}, @_);
+  } # _add_elem_array
 
-=head2 urls
-
-Return a reference to an array of urls.
-There is also a primary URL (C<url>).
-See also C<add_url>.
 
 =head2 add_url
 
 Add a URL to the list.
 
-=head2 related_urls, add_related_url, related_titles, add_related_title
+=cut
 
-Analgous to urls, these functions provide lists of related URLs
-and their titles.  These point to things the search engine thinks
-you might want.
+sub add_url { return shift->_add_elem_array('urls', @_); }
+
+=head2 urls
+
+Return a reference to the list of urls.
+There is also a primary URL (C<url>).
 
 =cut
 
 sub urls { return shift->_elem_array('urls', @_); }
-sub add_url { return shift->_add_elem_array('urls', @_); }
-sub related_urls { return shift->_elem_array('related_urls', @_); }
+
+=head2 add_related_url
+
+Add a URL to the related_url list.
+
+=cut
+
 sub add_related_url { return shift->_add_elem_array('related_urls', @_); }
-sub related_titles { return shift->_elem_array('related_titles', @_); }
+
+=head2 related_urls
+
+Return a reference to the list of related urls.
+
+=cut
+
+sub related_urls { return shift->_elem_array('related_urls', @_); }
+
+=head2 add_related_title
+
+Add a title to the list or related titles.
+
+=cut
+
 sub add_related_title { return shift->_add_elem_array('related_titles', @_); }
 
+=head2 related_titles
+
+Return a reference to the list of related titles.
+
+=cut
+
+sub related_titles { return shift->_elem_array('related_titles', @_); }
+
+
 =head2 title, description, score, change_date, index_date, size, raw
 
 Set or get attributes of the result.
@@ -160,11 +203,19 @@ Typical contents of these attributes:
 The title of the hit result (typically that provided by the 'TITLE'
 HTML tag).
 
+=cut
+
+sub title { return shift->_elem('title', @_); }
+
 =item description
 
 A brief description of the result, as provided (or not) by the search engine.
 Often the first few sentences of the document.
 
+=cut
+
+sub description { return shift->_elem('description', @_); }
+
 =item source
 
 Source is either the base url for this result (as listed on the search
@@ -176,6 +227,10 @@ result page said "found at CNN.com".
 This value is backend-specific; in fact very few backends set this
 value.
 
+=cut
+
+sub source { return shift->_elem('source', @_); }
+
 =item score
 
 A backend specific, numeric score of the search result.
@@ -183,6 +238,10 @@ The exact range of scores is search-engine specific.
 Usually larger scores are better, but this is no longer required.
 See normalized_score for a backend independent score.
 
+=cut
+
+sub score { return shift->_elem('score', @_); }
+
 =item normalized_score
 
 This is intended to be a backend-independent score of the search
@@ -192,14 +251,27 @@ indicate better quality results.
 This is not really implemented since no one has created an
 backend-independent ranking algorithm.
 
+=cut
+
+sub normalized_score { return shift->_elem('normalized_score', @_); }
+
 =item change_date
 
-When the result was last changed.
+When the result was last changed.  Typically this is the modification
+time of the destination web page.
+
+=cut
+
+sub change_date { return shift->_elem('change_date', @_); }
 
 =item index_date
 
 When the search engine indexed the result.
 
+=cut
+
+sub index_date { return shift->_elem('index_date', @_); }
+
 =item size
 
 The approximate size of the result, in bytes.  This is only an
@@ -207,6 +279,10 @@ approximation because search backends often report the size as
 "18.4K"; the best we can do with that number is return it as the value
 of 18.4 * 1024.
 
+=cut
+
+sub size { return shift->_elem('size', @_); }
+
 =item raw
 
 The raw HTML for the entire result.  Raw should be exactly the raw
@@ -220,24 +296,16 @@ dependent.  In fact, many backends do not even return it at all.
 
 =cut
 
-sub change_date { return shift->_elem('change_date', @_); }
-sub description { return shift->_elem('description', @_); }
-sub index_date { return shift->_elem('index_date', @_); }
-sub normalized_score { return shift->_elem('normalized_score', @_); }
 sub raw { return shift->_elem('raw', @_); }
-sub score { return shift->_elem('score', @_); }
-sub size { return shift->_elem('size', @_); }
-sub title { return shift->_elem('title', @_); }
 
-=head2 company, location, source
+=head2 Others
 
-More attributes of the result.
+More attributes of the result.  Backend-specific.
 
 =cut
 
 sub company { return shift->_elem('company', @_); }
 sub location { return shift->_elem('location', @_); }
-sub source { return shift->_elem('source', @_); }
 
 
 1;
